import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Input, InputNumber, notification, Modal } from 'antd'
import { EllipsisOutlined } from '@ant-design/icons'

import Utils from '@/utils/utils.js'
import MKEmitter from '@/utils/events.js'
import SubTable from '../subTable'
import './index.scss'

class DetailLine extends React.Component {
  state = {
    bill_count: 0,
    unitprice: 0,
    amount_line: 0,
    visible: false
  }

  UNSAFE_componentWillMount() {
    let { line } = this.props

    this.setState({
      bill_count: line.bill_count || '',
      unitprice: line.unitprice || '',
      amount_line: line.amount_line || ''
    })
  }

  onChange = (value, key, nextkey) => {
    let line = {...this.props.line}

    if (['bill_count', 'unitprice', 'amount_line'].includes(key)) {
      line[key] = value || 0
      if (isNaN(line[key])) {
        line[key] = 0
      }
      if (line[key]) {
        if (key === 'bill_count') {
          line[key] = Math.round(line[key] * 10000000000) / 10000000000
          if (line.unitprice) {
            line.amount_line = Math.round(line.unitprice * line.bill_count * 100) / 100
          }
        } else if (key === 'unitprice') {
          line[key] = Math.round(line[key] * 10000000000) / 10000000000
          if (line.bill_count) {
            line.amount_line = Math.round(line.unitprice * line.bill_count * 100) / 100
          }
        } else if (key === 'amount_line') {
          line[key] = Math.round(line[key] * 100) / 100
          if (line.bill_count) {
            line.unitprice = Math.round(line.amount_line / line.bill_count * 10000000000) / 10000000000
          } else if (line.unitprice) {
            line.bill_count = Math.round(line.amount_line / line.unitprice * 10000000000) / 10000000000
          }
        }
      } else if (key === 'amount_line') {
        line.bill_count = 0
      }

      if (line.amount_line) {
        line.tax_amount = line.amount_line - line.amount_line / (line.tax_rate + 1)
        line.tax_amount = Math.round(line.tax_amount * 100) / 100
      } else {
        line.tax_amount = 0
      }
    } else {
      line[key] = value
    }

    this.setState({
      bill_count: line.bill_count || '',
      unitprice: line.unitprice || '',
      amount_line: line.amount_line || ''
    })

    this.props.changeLine(line, key)

    if (nextkey) {
      let node = document.getElementById(nextkey)
      if (node) {
        if (node.select) {
          node.select()
        } else if (node.focus) {
          node.focus()
        }
      }
    }
  }

  onSkip = (key) => {
    let node = document.getElementById(key)
    if (node) {
      if (node.select) {
        node.select()
      } else if (node.focus) {
        node.focus()
      }
    }
  }

  render() {
    const { line, delLine, trigger, field, pid } = this.props
    const { bill_count, unitprice, amount_line } = this.state
    
    return <div className="mk-tr active">
      <div className="mk-td">
        <div className="mk-input">{line.productname || ''}<EllipsisOutlined onClick={trigger}/></div>
      </div>
      <div className="mk-td">
        <Input defaultValue={line.spec || ''} autoFocus={field === 'spec'} onChange={(e) => this.onChange(e.target.value, 'spec')} onPressEnter={() => this.onSkip(pid + 'mk-invoice-unit')}/>
      </div>
      <div className="mk-td">
        <Input id={pid + 'mk-invoice-unit'} defaultValue={line.unit || ''} autoFocus={field === 'unit'} onChange={(e) => this.onChange(e.target.value, 'unit')} onPressEnter={() => this.onSkip(pid + 'mk-invoice-billcount')}/>
      </div>
      <div className="mk-td">
        <InputNumber id={pid + 'mk-invoice-billcount'} value={bill_count} autoFocus={field === 'bill_count'} formatter={value => `${value}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')} parser={value => value.replace(/,*/g, '')} onChange={(val) => this.setState({bill_count: val})} onPressEnter={() => this.onChange(bill_count, 'bill_count', pid + 'mk-invoice-unitprice')} onBlur={() => this.onChange(bill_count, 'bill_count')}/>
      </div>
      <div className="mk-td">
        <InputNumber id={pid + 'mk-invoice-unitprice'} value={unitprice} autoFocus={field === 'unitprice'} formatter={value => `${value}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')} parser={value => value.replace(/,*/g, '')} onChange={(val) => this.setState({unitprice: val})} onPressEnter={() => this.onChange(unitprice, 'unitprice', pid + 'mk-invoice-amount')} onBlur={() => this.onChange(unitprice, 'unitprice')}/>
      </div>
      <div className="mk-td">
        <InputNumber id={pid + 'mk-invoice-amount'} value={amount_line} autoFocus={field === 'amount_line'} formatter={value => `${value}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')} parser={value => value.replace(/,*/g, '')} onChange={(val) => this.setState({amount_line: val})} onPressEnter={() => this.onChange(amount_line, 'amount_line')} onBlur={() => this.onChange(amount_line, 'amount_line')}/>
      </div>
      <div className="mk-td mk-right">{line.tax_name}</div>
      <div className="mk-td mk-right">{line.tax_amount.toFixed(2).replace(/\B(?=(\d{3})+(?!\d))/g, ',')} <span className="del-line" onClick={(e) => delLine(line.uuid, e)}></span> </div>
    </div>
  }
}

class InvoiceTable extends Component {
  static propTpyes = {
    config: PropTypes.object,
    timestamp: PropTypes.string,
    read_only: PropTypes.any,
    data: PropTypes.any,
    onChange: PropTypes.func
  }

  state = {
    data: [],
    editKey: '',
    key: '',
    total: {}
  }

  UNSAFE_componentWillMount () {
    const { data } = this.props

    let _data = fromJS(data).toJS()
    if (!_data.length) {
      _data = [{uuid: Utils.getguid(), productname: '', spec: '', unit: '', bill_count: '', unitprice: 0, amount_line: 0, tax_rate: '', tax_name: '', tax_amount: 0}]
    }

    this.setState({
      data: _data
    }, () => {
      this.getTotal(_data)
    })
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    MKEmitter.addListener('resetDetails', this.resetDetails)
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    if (this.props.timestamp !== nextProps.timestamp) {
      let _data = fromJS(nextProps.data).toJS()
      if (!_data.length) {
        _data = [{uuid: Utils.getguid(), productname: '', spec: '', unit: '', bill_count: '', unitprice: 0, amount_line: 0, tax_rate: '', tax_name: '', tax_amount: 0}]
      }

      this.setState({
        data: _data
      }, () => {
        this.getTotal(_data)
      })
    }
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('resetDetails', this.resetDetails)
  }
  
  changeMoneyToChinese = (money) => {
    let cnNums = ['零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖']
    let cnIntRadice = ['', '拾', '佰', '仟']
    let cnIntUnits = ['', '万', '亿', '兆']
    let cnDecUnits = ['角', '分', '毫', '厘']
    let cnInteger = '整'
    let cnIntLast = '元'
    let maxNum = 999999999999999.9999 // 最大处理的数字
    let IntegerNum = null
    let DecimalNum = null
    let ChineseStr = ''
    let parts = null // 分离金额后用的数组，预定义
    let Symbol = ''  // 正负值标记

    if (money === 0) return ''

    if (money >= maxNum) return '超出最大处理数字'

    if(money < 0) {
      money = -money
      Symbol = '负'       
    }
    money = money.toString() // 转换为字符串
    if (money.indexOf('.') === -1) {
      IntegerNum = money
      DecimalNum = ''
    } else {
      parts = money.split('.')
      IntegerNum = parts[0]
      DecimalNum = parts[1].substr(0, 4)
    }

    if (parseInt(IntegerNum, 10) > 0) { // 获取整型部分转换
      let zeroCount = 0
      let IntLen = IntegerNum.length
      for (let i = 0; i < IntLen; i++) {
        let n = IntegerNum.substr(i, 1)
        let p = IntLen - i - 1
        let q = p / 4
        let m = p % 4
        
        if (n === '0') {
          zeroCount++
        } else {
          if (zeroCount > 0) {
            ChineseStr += cnNums[0]
          }
          zeroCount = 0 // 归零
          ChineseStr += cnNums[parseInt(n)] + cnIntRadice[m]
        }

        if (m === 0 && zeroCount < 4) {
          ChineseStr += cnIntUnits[q]
        }
      }
      ChineseStr += cnIntLast
    }

    if (DecimalNum !== '') { // 小数部分
      let decLen = DecimalNum.length

      for (let i = 0; i < decLen; i++) {
        let n = DecimalNum.substr(i, 1)
        if (n !== '0') {
          ChineseStr += cnNums[Number(n)] + cnDecUnits[i]
        }
      }
    }
    if (ChineseStr === '') {
      ChineseStr += cnNums[0] + cnIntLast + cnInteger
    } else if (DecimalNum === '') {
      ChineseStr += cnInteger
    }

    ChineseStr = Symbol + ChineseStr
    
    return ChineseStr
  }

  getTotal = (data) => {
    let price = 0
    let tax = 0

    data.forEach(item => {
      if (!item.productcode) return

      price += item.amount_line * 100
      tax += item.tax_amount * 100
    })

    price = price / 100
    tax = tax / 100

    this.setState({total: {price, tax, sum: price, sumName: this.changeMoneyToChinese(price)}})
  }

  resetDetails = (data) => {
    let _data = fromJS(data).toJS()

    if (!_data.length) {
      _data = [{uuid: Utils.getguid(), productname: '', spec: '', unit: '', bill_count: '', unitprice: 0, amount_line: 0, tax_rate: '', tax_name: '', tax_amount: 0}]
    }

    this.setState({data: _data}, () => {
      this.getTotal(_data)
    })
  }

  addLine = () => {
    const { data } = this.state

    let line = {uuid: Utils.getguid(), productname: '', spec: '', unit: '', bill_count: '', unitprice: 0, amount_line: 0, tax_rate: '', tax_name: '', tax_amount: 0}

    this.setState({data: [...data, line]})
  }

  delLine = (uuid, e) => {
    const { data } = this.state

    e.stopPropagation()
    
    if (data.length === 1) {
      notification.warning({
        top: 92,
        message: '至少保留一行明细！',
        duration: 3
      })
      return
    }

    let _data = data.filter(item => item.uuid !== uuid)

    this.setState({data: _data}, () => {
      this.getTotal(_data)
    })
    this.props.onChange(_data)
  }

  changeLine = (record) => {
    const { editKey, data } = this.state

    let _data = data.map(item => {
      if (item.uuid === editKey) {
        return record
      } else {
        return item
      }
    })


    this.setState({data: _data}, () => {
      this.getTotal(_data)
    })
    this.props.onChange(_data)
  }

  checkLine = (uuid, key, e) => {
    const { read_only } = this.props

    e && e.stopPropagation()

    if (read_only) return
    
    this.setState({editKey: uuid, key: key || ''}, () => {
      if (key === 'productname') {
        this.setState({visible: true})
      }
    })
  }

  changeDetail = (prod) => {
    const { editKey, data } = this.state

    let _data = data.map(item => {
      if (item.uuid === editKey) {
        item.productname = prod.productname
        item.spec = prod.spec
        item.unit = prod.unit
        item.unitprice = prod.unitprice
        item.tax_rate = prod.tax_rate || 0
        item.tax_name = prod.tax_rate * 100 + '%'
        item.free_tax_mark = prod.free_tax_mark || ''
        item.vat_special_management = prod.vat_special_management || ''
        item.tax_item = prod.tax_item || ''
        item.tax_method = prod.tax_method || ''

        if (prod.vat_special_management && prod.free_tax_mark === 'true') {
          item.tax_name = prod.vat_special_management
        }

        item.productcode = prod.productcode
        item.Description = prod.Description
        item.tax_classify_code = prod.tax_classify_code
        item.tax_classify_name = prod.tax_classify_name

        if (item.bill_count && item.unitprice) {
          item.amount_line = Math.round(item.unitprice * item.bill_count * 100) / 100
        }
        if (item.amount_line) {
          item.tax_amount = item.amount_line - item.amount_line / (item.tax_rate + 1)
          item.tax_amount = Math.round(item.tax_amount * 100) / 100
        } else {
          item.tax_amount = 0
        }
      }

      return item
    })

    this.setState({data: _data, editKey: '', visible: false}, () => {
      this.setState({editKey: editKey})
      this.getTotal(_data)
    })
    this.props.onChange(_data)
  }

  render() {
    const { config, tax_type } = this.props
    const { editKey, key, data, total, visible } = this.state

    return (
      <div className="detail-wrap">
        <span className="plus-line" onClick={this.addLine}></span>
        <div className="mk-th">
          <div className="mk-td">货物或应税劳务、服务名称</div>
          <div className="mk-td">规格型号</div>
          <div className="mk-td">单位</div>
          <div className="mk-td">数量</div>
          <div className="mk-td">单价（含税）</div>
          <div className="mk-td">金额（含税）</div>
          <div className="mk-td">税率</div>
          <div className="mk-td">税额</div>
        </div>
        {data.map(item => {
          if (editKey === item.uuid) {
            return <DetailLine key={item.uuid} pid={config.uuid} line={item} field={key} changeLine={this.changeLine} delLine={this.delLine} trigger={() => this.setState({visible: true})}/>
          }

          return <div className="mk-tr" key={item.uuid} onClick={() => this.checkLine(item.uuid)}>
            <div className="mk-td mk-left" onClick={(e) => this.checkLine(item.uuid, item.productname ? '' : 'productname', e)}>{item.productname || ''}</div>
            <div className="mk-td mk-left" onClick={(e) => this.checkLine(item.uuid, 'spec', e)}>{item.spec || ''}</div>
            <div className="mk-td mk-left" onClick={(e) => this.checkLine(item.uuid, 'unit', e)}>{item.unit || ''}</div>
            <div className="mk-td mk-right" onClick={(e) => this.checkLine(item.uuid, 'bill_count', e)}>{`${item.bill_count || ''}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}</div>
            <div className="mk-td mk-right" onClick={(e) => this.checkLine(item.uuid, 'unitprice', e)}>{`${item.unitprice || ''}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}</div>
            <div className="mk-td mk-right" onClick={(e) => this.checkLine(item.uuid, 'amount_line', e)}>{`${item.amount_line || ''}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}</div>
            <div className="mk-td mk-right">{item.tax_name}</div>
            <div className="mk-td mk-right">{item.tax_amount.toFixed(2).replace(/\B(?=(\d{3})+(?!\d))/g, ',')}<span className="del-line" onClick={(e) => this.delLine(item.uuid, e)}></span></div>
          </div>
        })}
        <div className="mk-total">
          <div className="mk-td">合计</div>
          <div className="mk-td"></div>
          <div className="mk-td"></div>
          <div className="mk-td"></div>
          <div className="mk-td"></div>
          <div className="mk-td">￥{`${total.price}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}</div>
          <div className="mk-td"></div>
          <div className="mk-td">￥{`${total.tax}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}</div>
        </div>
        <div className="mk-upcase">
          <div className="mk-td">价税合计（大写）</div>
          <div className="mk-td">{total.sumName}</div>
          <div className="mk-td">（小写）￥{`${total.sum}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}</div>
        </div>
        <Modal
          title="商品信息"
          visible={visible}
          width="75vw"
          maskClosable={false}
          onCancel={() => { this.setState({ visible: false }) }}
          footer={null}
        >
          <SubTable config={config} tax_type={tax_type} onChange={this.changeDetail}/>
        </Modal>
      </div>
    )
  }
}

export default InvoiceTable